#ifndef INC_INILINE_H
#define INC_INILINE_H
//-----------------------------------------------------------------------------
/*!
\file IniLine.h
\brief
\if english
  Header for Line
\endif

\if german
  Header für Line
\endif
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the IniFile module.

IniFile is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General
Public License as published by the Free Software Foundation; either version 3 of
the License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "IniFileTypes.h"
#include "IniLineFactory.h"
//-----------------------------------------------------------------------------
#ifdef _MSC_VER
  /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
  #pragma warning( disable : 4251 )
#endif
//-----------------------------------------------------------------------------
namespace ini
{
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Is a comment line, but also superclass for Section and Key.
  \endif

  \if german
    Ist eine Kommentartzeile, aber auch Oberklasse für Section und Key.
  \endif

  \author Helmut Jakoby
  \copyright © 1995 Helmut Jakoby
  */
  class __ini_file_export_dll Line
  {
    private:
      //============== Friends
      //----------------------------------------------------------------------
      friend class LineFactory;
      friend class File;
      friend class Section;
      //----------------------------------------------------------------------

    protected:
      //============== Attribute ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        The text (one line of the INI file) of the entry is stored in this attribute.
      \endif
    
      \if german
        In diesem Attribut wird die Text (eine Zeile der INI-Datei) des Eintrags abgelegt.
      \endif
      */
      std::string m_sText;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The type of the entry.
      \endif
    
      \if german
        Der Typ des Eintrags.
      \endif
      */
      LINE_TYPE m_eLineType;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Helper class, returns among other things a type-appropriate entry.
      \endif
    
      \if german
        Helferklasse, liefert u.a einen typegerechten Eintrag.
      \endif
      */
      LineFactory m_LineFactory;
      //-----------------------------------------------------------------------

    public:
      //============== Konstruktoren ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing.

        \param[in] sCharLine The text of the entry.
      \endif
    
      \if german
        Konstruktor mit Parameterübergabe.

        \param[in] sCharLine Der Text des Eintrags.
      \endif
      */
      Line( const std::string & sCharLine );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor, memory being purged.
      \endif
    
      \if german
        Destruktor, Speicher wird aufgeräumt.
      \endif
      */
      virtual ~Line();
      //-----------------------------------------------------------------------

    protected:
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor with parameter passing is not publicly available.

        \param[in] sCharLine The text of the entry.
        \param[in] eLineType The desired type of the entry.
      \endif
    
      \if german
        Dieser Konstruktor mit Parameterübergabe steht nicht öffendlich zur Verfügung.

        \param[in] sCharLine Der Text des Eintrags.
        \param[in] eLineType Der gewünschte Typ des Eintrags.
      \endif
      */
      Line( const std::string & sCharLine, LINE_TYPE eLineType );
      //-----------------------------------------------------------------------
      /*!
      \if english
        The standard constructor is not publicly available.
      \endif
    
      \if german
        Standartkonstruktor steht nicht öffendlich zur Verfügung.
      \endif
      */
      Line();
      //-----------------------------------------------------------------------

    private:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif
    
      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      inline Line( const Line & );
      //-----------------------------------------------------------------------

    public:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the text of the INI entry.
        \return The text.
      \endif
    
      \if german
        Liefert den Text des INI-Eintrags.
        \return Der Text.
      \endif
      */
      std::string getText() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sets the text of the INI entry.
        \param[in] sCharLine The text as string.
      \endif
    
      \if german
        Setzt den Text des INI-Eintrags.
        \param[in] sCharLine Der Text als String.
      \endif
      */
      void setText( std::string sCharLine );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the type of the entry.
        \return The type as enumaration.
      \endif
    
      \if german
        Liefert den Typ des Eintrags.
        \return Der Typ als Enumaration.
      \endif
      */
      LINE_TYPE getType() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the length of the string from #m_sText plus EOL.
        \return The length of the entry text.
      \endif
    
      \if german
        Liefert die Länge des Strings aus #m_sText plus EOL.
        \return Die Länge des Eintragstextes.
      \endif
      */
      std::size_t getLineSizeWithEOL() const;
      //-----------------------------------------------------------------------

    protected:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the text of the INI entry as an EOL terminated string.
        \return The text with a line break at the end.
      \endif
    
      \if german
        Liefert den Text des INI-Eintrags als einen EOL-terminierten String.
        \return Der Text mit Zeilenumbruch am Ende.
      \endif
      */
      char * getTextWithEOL() const;
      //-----------------------------------------------------------------------

    private:
      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The assignment operator is not available.
      \endif
    
      \if german
        Der Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      Line & operator= ( const Line & );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
    #pragma warning( default : 4251 )
  #endif
  //---------------------------------------------------------------------------
} // namespace ini
//-----------------------------------------------------------------------------
#endif
