//-----------------------------------------------------------------------------
/*!
\file IniFile.cpp
\brief Source for File
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the IniFile module.

IniFile is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General
Public License as published by the Free Software Foundation; either version 3 of
the License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
#include "IniFile.h"
#include <fstream>
#include <cstring>
#include "EuBlob.h"
#include "EuException.h"
//-----------------------------------------------------------------------------
#ifdef _DEBUG_MEMORY_SPY
  #include <MilDebugNew.h>
#endif
//-----------------------------------------------------------------------------
namespace ini
{
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    //This function or variable may be unsafe
    #pragma warning( disable : 4996 )
  #endif
  //---------------------------------------------------------------------------
  File::File( const std::string & rsFullFileName, TdOpenInifileMode openMode )
  : m_sFileName(rsFullFileName),
    m_LineList(ptl::RESPONSIBLE|ptl::UNIQUE)
  {
    int  t_iErr = 0;

    if ( ! m_sFileName.empty() )
    {
      if ( openMode )
      {
        t_iErr = this->refresh();
      }
    }
    else
    {
      t_iErr = NO_FILENAME;
    }
    if ( t_iErr )
    {
      m_LineList.removeAll();
      throw eut::ErrorNException( "ini::File instantiation error!", t_iErr, __FILE__, __LINE__ );
    }
  }
  //---------------------------------------------------------------------------
  File::File()
  : m_LineList(ptl::RESPONSIBLE|ptl::UNIQUE)
  {
  }
  //---------------------------------------------------------------------------
  File::~File()
  {
  }
  //---------------------------------------------------------------------------
  int File::refresh()
  {
    int             t_iErr = 0;
    std::ifstream   t_InputFile;
    Section       * t_pCurrentSection = NULL_PTR;  // die bekommt alles wenn es eine gibt
    Line          * t_pNewItem = NULL_PTR;

    if ( ! m_sFileName.empty() )
    {
      t_InputFile.open( m_sFileName.c_str() );
      if ( t_InputFile.good() )
      {
        std::string     t_sInputBuffer;
        
        // alle Einträge aus der Liste entfernen
        m_LineList.removeAll();
        // Einträge erstellen
        while ( ! t_iErr && std::getline( t_InputFile, t_sInputBuffer) )
        {
          t_iErr = m_LineFactory.getIniLine( t_pNewItem, t_sInputBuffer );
          if ( ! t_iErr )
          {
            if ( t_pCurrentSection && !( t_pNewItem->getType() == IniSECTION ) )
            {
              t_iErr = t_pCurrentSection->insertSectionLine( t_pNewItem );
            }
            else
            {
              ///*
              // Wenn Section, prüfen ob schon drinn, dann Fehler!
              if ( t_pNewItem->getType() == IniSECTION )
              {
                Line        * t_pTestItem = NULL_PTR;
                std::string   sNewItemText = t_pNewItem->getText();

                this->getLine( t_pTestItem, & sNewItemText );
                if ( t_pTestItem )
                {
                  delete t_pNewItem;
                  t_iErr = SECTION_ALLREADY_EXISTING;
                }
              }
              if ( ! t_iErr )
              //*/
              {
                // kommt noch in INI-Datei ohne Sektion
                t_iErr = this->insertLine( t_pNewItem );
                if ( t_pNewItem->getType() == IniSECTION )
                {
                  t_pCurrentSection = dynamic_cast<Section*> (t_pNewItem);
                  if ( !t_pCurrentSection )
                  {
                    t_iErr = CAST;
                  }
                }
              }
            }
          }
        }
      }
      else
      {
        t_iErr = READ_ERROR;
      }
    }
    else
    {
      t_iErr = NO_FILENAME;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: implicit conversion changes signedness: A to B [-Wsign-conversion]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wsign-conversion"
  #endif
  //---------------------------------------------------------------------------
  int File::store()
  {
    int             t_iErr = 0;
    eut::Blob     * t_pBlob = NULL_PTR;
    char          * t_pTempBuffer = NULL_PTR;
    std::ofstream   t_OutputFile;

    if ( ! m_sFileName.empty() )
    {
      // Einträge in Blob erstellen
      t_iErr = this->getAsBlob( t_pBlob );
      if ( !t_iErr )
      {
        t_OutputFile.open( m_sFileName.c_str(), std::ios_base::binary );
        if ( t_OutputFile )
        {
          std::size_t   t_nBlobSize = t_pBlob->getData( t_pTempBuffer, false /* Keine Kopie */);
          t_OutputFile.write( t_pTempBuffer, t_nBlobSize );
          t_iErr = t_OutputFile.fail();
          // Blob vergessen
        }
        else
        {
          t_iErr = WRITE_ERROR;
        }
        delete t_pBlob;
      }
    }
    else
    {
      t_iErr = NO_FILENAME;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: implicit conversion changes signedness: A to B [-Wsign-conversion]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  void File::setFullFileName( const std::string & rsFullFileName )
  {
    m_sFileName = rsFullFileName;
  }
  //---------------------------------------------------------------------------
  std::string File::getFileName()
  {
    return m_sFileName;
  }
  //---------------------------------------------------------------------------
  int File::getLine( Line *& prRetVal, std::size_t nPosition, ptl::SeekMode eMode )
  {
    return m_LineList.get( prRetVal, nPosition, eMode );
  }
  //---------------------------------------------------------------------------
  int File::insertLine(  Line * pIniLine,
                         ptl::InsertPosition eInsertPosition,
                         ptl::VH eBeforeBehind )
  {
    return m_LineList.insert( pIniLine, eInsertPosition, eBeforeBehind );
  }
  //---------------------------------------------------------------------------
  int File::removeLine( ptl::ElementPosition eFirstLastCurrent )
  {
    return m_LineList.remove( eFirstLastCurrent );
  }
  //---------------------------------------------------------------------------
  int File::getLine(  Line *& prRetVal,
                      std::string * psSection,
                      std::string * psKey )
  {
    int           t_iErr = 0;
    Line        * t_pIniLine = NULL_PTR;
    LineFactory   t_IniLineFactory;

    prRetVal = NULL_PTR;
    // alle Strings trimmen
    if ( psSection ) t_IniLineFactory.trim( *psSection );
    if ( psKey ) t_IniLineFactory.trim( *psKey );
    if ( ! psSection )
    {
      t_iErr = this->getLine( t_pIniLine, 0, ptl::START );
      if ( ! t_iErr )
      {
        do
        {
          // Nachsehen ob IniVARIABLE
          if  ( t_pIniLine->getType() == IniVARIABLE )
          {
            Key * t_pKey = dynamic_cast<Key*>( t_pIniLine );
            if ( t_pKey )
            {
              if ( psKey && *psKey == t_pKey->getKeyName() )
              prRetVal = t_pIniLine;
            }
            else
            {
              t_iErr = CAST;
            }
          }
          if ( ! t_iErr && ! prRetVal)
          {
            t_iErr = this->getLine( t_pIniLine, 1, ptl::CURRENT );
          }
        } while ( ! t_iErr && ! prRetVal );
      }
      if ( t_iErr == ptl::RANGE )
      {
        t_iErr = KEY_NOT_EXISTING;
      }
    }
    else
    {
      // an die richtige Sektion
      t_iErr = this->getLine( t_pIniLine, 0, ptl::START );
      if ( ! t_iErr )
      {
        do
        {
          if ( t_pIniLine->getType() == IniSECTION )
          {
            Section * t_pSektion = dynamic_cast<Section*>( t_pIniLine );
            if ( t_pSektion )
            {
              // Vergleichsstrings zusammenbauen, wenn nötig
              std::string   t_sCompairSectionText = *psSection;

              if ( t_sCompairSectionText.size() && t_sCompairSectionText.operator[](0) != '[' )
              {
                // Eintragsstring erstellen -> "[" + szSection + ']'
                std::string t_sTemp = "[";
                t_sTemp += t_sCompairSectionText;
                t_sTemp += "]";
                t_sCompairSectionText = t_sTemp;
              }

              if ( t_sCompairSectionText == t_pSektion->getText() )
              {
                if ( psKey )
                {
                  t_iErr = t_pSektion->getKeyLine( prRetVal, *psKey );
                }
                else
                {
                  prRetVal = t_pSektion;
                }
              }
            }
            else
            {
              t_iErr = CAST;
            }
          }
          if ( ! t_iErr && ! prRetVal)
          {
            t_iErr = this->getLine( t_pIniLine, 1, ptl::CURRENT );
          }
        } while ( ! t_iErr && ! prRetVal);
      }
      if ( t_iErr == ptl::RANGE )
      {
        t_iErr = SECTION_NOT_EXISTING;
      }
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  int File::getKeyValue(  std::string & rsValue,
                          std::string * psSection,
                          std::string * psKey )
  {
    int           t_iErr = 0;
    Line        * t_pIniLine = NULL_PTR;
    LineFactory   t_IniLineFactory;

    rsValue = "";
    t_iErr = this->getLine( t_pIniLine, psSection,  psKey );
    if ( ! t_iErr )
    {
      Key * t_pKey = dynamic_cast<Key*>( t_pIniLine );
      if ( t_pKey )
      {
        rsValue = t_pKey->getKeyValue();
      }
      else
      {
        t_iErr = CAST;
      }
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: dereferencing type-punned pointer will break strict-aliasing rules [-Wstrict-aliasing]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wstrict-aliasing"
    // Warnung: use of old-style cast [-Wold-style-cast]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wold-style-cast"
  #endif
  //---------------------------------------------------------------------------
  int File::setKeyValue(  const std::string & rsSection,
                          const std::string & rsKey,
                          const std::string & rsValue )
  {
    int           t_iErr = 0;
    std::string   t_sSection,
                  t_sKey,
                  t_sValue,
                  t_sTemp;
    std::string * t_psSection = NULL_PTR,
                * t_psKey = NULL_PTR,
                * t_psValue = NULL_PTR;
    LineFactory   t_IniLineFactory;
    Line        * t_pIniLine = NULL_PTR;
    Section     * t_pSektion = NULL_PTR;

    if ( ! rsSection.empty() )
    {
      t_sSection = rsSection;
      t_psSection = & t_sSection;
    }
    if ( ! rsKey.empty() )
    {
      t_sKey = rsKey;
      t_psKey = & t_sKey;
    }
    if ( ! rsValue.empty() )
    {
      t_sValue = rsValue;
      t_psValue = & t_sValue;
    }
    // alle Strings trimmen
    if ( t_psSection ) t_IniLineFactory.trim( * t_psSection );
    if ( t_psKey ) t_IniLineFactory.trim( * t_psKey );
    if ( t_psValue ) t_IniLineFactory.trim( * t_psValue );

    // prüfen ob Sektion gemeint ist
    if ( t_psSection )
    {
      // Sektion zusammenbauen, wenn nötig
      if ( t_psSection->operator[](0) != '[' )
      {
        // Eintragsstring erstellen -> "[" + szSection + ']'
        t_sTemp = "[";
        t_sTemp += *t_psSection;
        t_sTemp += "]";
        *t_psSection = t_sTemp;
      }
      // Sektion holen
      t_iErr = this->getLine( (Line*&)t_pSektion, t_psSection );
      if ( t_iErr == SECTION_NOT_EXISTING )
      {
        // neue Sektion anlegen
        #ifdef _DEBUG_MEMORY_SPY
          t_pSektion = new Section( *t_psSection ); // Wenn Fehler in "new", wird 0 retourgeliefert
        #else
          t_pSektion = new (std::nothrow) Section( *t_psSection ); // Wenn Fehler in "new", wird 0 retourgeliefert
        #endif
        if ( t_pSektion )
        {
          t_iErr = this->insertLine( t_pSektion );
          if ( t_iErr )
          {
            delete t_pSektion;
            t_pSektion = NULL_PTR;
          }
        }
      }
    }
    if ( ! t_iErr && t_psKey && t_psValue )
    {
      if ( t_pSektion )
      {
        t_iErr = t_pSektion->getKeyLine( t_pIniLine, *t_psKey );
      }
      else
      {
        t_iErr = this->getLine( t_pIniLine, NULL_PTR, t_psKey );
      }
      // Wenn Eintrag, dann ist dieses eine Variable ergo Wert setzen
      if ( t_pIniLine )
      {
        Key * t_pKey = dynamic_cast<Key*>( t_pIniLine );
        if ( t_pKey )
        {
          t_pKey->setKeyValue( *t_psValue );
        }
        else
        {
          t_iErr = CAST;
        }
      }
      else // kein Eintrag gefunden
      {
        // Eintragsstring erstellen -> szKey + '=' + szValue
        t_sTemp = *t_psKey;
        t_sTemp += "=";
        t_sTemp += *t_psValue;
        // Eintrag generieren
        t_iErr = t_IniLineFactory.getIniLine( t_pIniLine, t_sTemp );
        if ( ! t_iErr )
        {
          if ( t_pSektion )
          {
            t_iErr = t_pSektion->insertSectionLine( t_pIniLine );
          }
          else
          {
            t_iErr = this->insertLine( t_pIniLine );
          }
        }
      }
    }
    else
    {
      t_iErr = PARAMETER_NULL_FORBIDDEN ;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  int File::removeKey( const std::string & rsSection, const std::string & rsKey )
  {
    int           t_iErr = 0;
    std::string   t_sSection,
                  t_sKey,
                  t_sTemp;
    std::string * t_psSection = NULL_PTR,
                * t_psKey = NULL_PTR;
    LineFactory   t_IniLineFactory;
    Line        * t_pIniLine = NULL_PTR;
    Section      * t_pSektion = NULL_PTR;

    if ( ! rsSection.empty() )
    {
      t_sSection = rsSection;
      t_psSection = & t_sSection;
    }
    if ( ! rsKey.empty() )
    {
      t_sKey = rsKey;
      t_psKey = & t_sKey;
    }
    // alle Strings trimmen
    if ( t_psSection ) t_IniLineFactory.trim( * t_psSection );
    if ( t_psKey ) t_IniLineFactory.trim( * t_psKey );

    // prüfen ob Sektion gemeint ist
    if ( t_psSection )
    {
      // Sektion zusammenbauen, wenn nötig
      if ( t_psSection->operator[](0) != '[' )
      {
        // Eintragsstring erstellen -> "[" + szSection + ']'
        t_sTemp = "[";
        t_sTemp += *t_psSection;
        t_sTemp += "]";
        *t_psSection = t_sTemp;
      }
      // Sektion holen
      t_iErr = this->getLine( (Line*&) t_pSektion, t_psSection );
      if ( ! t_iErr && ! t_psKey )
      {
        t_iErr = this->removeLine();
      }
    }
    if ( ! t_iErr && t_psKey )
    {
      if ( t_pSektion )
      {
        t_iErr = t_pSektion->getKeyLine( t_pIniLine, *t_psKey );
        if ( ! t_iErr )
        {
          t_iErr = t_pSektion->removeSectionLine();
        }
      }
      else
      {
        t_iErr = this->getLine(t_pIniLine, NULL_PTR, t_psKey);
        if ( !  t_iErr )
        {
          t_iErr = this->removeLine();
        }
      }
    }
    else
    {
      t_iErr = PARAMETER_NULL_FORBIDDEN ;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: dereferencing type-punned pointer will break strict-aliasing rules [-Wstrict-aliasing]
    #pragma GCC diagnostic pop
    // Warnung: use of old-style cast [-Wold-style-cast]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  int File::getAsBlob( eut::Blob *& prBlob )
  {
    int     t_iErr = 0;
    Line  * t_pIniLine = NULL_PTR;

    // Einträge in Blob erstellen
    prBlob = new eut::Blob();
    t_iErr = this->getLine( t_pIniLine, 0, ptl::START );
    if ( ! t_iErr )
    {
      do 
      {
        // wenn Sektion; die kann selbst
        Section * t_pSektion = dynamic_cast<Section*>(t_pIniLine);
        if (t_pSektion)
        {
          t_iErr = t_pSektion->insertSectionInBlob( * prBlob );
        }
        else
        {
          char  * t_pTempBuffer = t_pIniLine->getTextWithEOL();
          t_iErr = prBlob->insert( t_pTempBuffer,
                                   t_pIniLine->getLineSizeWithEOL(),
                                   prBlob->getSize() + 1
                         );
          delete[] t_pTempBuffer;
        }
        if ( ! t_iErr )
        {
          t_iErr = this->getLine(t_pIniLine, 1, ptl::CURRENT);
        }
      } while ( ! t_iErr );
    }
    if ( t_iErr == ptl::RANGE )
    {
      t_iErr = 0;
    }
    if ( t_iErr )
    {
      delete prBlob;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  int File::makeLines( eut::Blob * pBlob )
  {
    int           t_iErr = 0;
    char        * t_szBuffer = NULL_PTR;
    std::size_t   t_nEndPosition = 0;
    Section     * t_pCurrentSection = NULL_PTR;  // die bekommt alles wenn es eine gibt
    Line        * t_pNewItem = NULL_PTR;

    // Zeile holen
    if ( pBlob->findForward( t_nEndPosition, ini::g_sEOL.c_str(), ini::g_sEOL.length() ) )
    {
      std::size_t   t_nCurrentPosition = 1;
      
      // Wenn gefunden
      do
      {
        if ( t_nEndPosition > t_nCurrentPosition )
        {
          t_iErr = pBlob->getData( t_szBuffer, t_nCurrentPosition, t_nEndPosition - t_nCurrentPosition );
        }
        if ( ! t_iErr )
        {
          char  * t_szIniLine = NULL_PTR;

          t_szIniLine = new char [(t_nEndPosition - t_nCurrentPosition) + 1];
          std::memcpy(t_szIniLine, t_szBuffer, (t_nEndPosition - t_nCurrentPosition));
          t_szIniLine[(t_nEndPosition - t_nCurrentPosition)] = 0x00;
          // EintragsObjekt holen
          t_iErr = m_LineFactory.getIniLine( t_pNewItem, t_szIniLine );
          if ( ! t_iErr )
          {
            if (t_pCurrentSection && !(t_pNewItem->getType() == IniSECTION))
            {
              t_iErr = t_pCurrentSection->insertSectionLine(t_pNewItem);
            }
            else
            {
              // kommt noch in INI-Datei ohne Sektion
              t_iErr = this->insertLine(t_pNewItem);
              if (t_pNewItem->getType() == IniSECTION)
              {
                t_pCurrentSection = dynamic_cast<Section*> (t_pNewItem);
                if ( ! t_pCurrentSection )
                {
                  t_iErr = CAST;
                }
              }
            }
          }
          t_nCurrentPosition = t_nEndPosition + ini::g_sEOL.length();
          if ( t_szBuffer )
          {
            delete[] t_szBuffer;
            t_szBuffer = NULL_PTR;
          }
          if ( t_szIniLine )
          {
            delete[] t_szIniLine;
            t_szIniLine = NULL_PTR;
          }
        }
      } while ( ! t_iErr && ( pBlob->findForwardNext( t_nEndPosition, ini::g_sEOL.c_str(), ini::g_sEOL.length() ) ) );
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
      #pragma warning( default : 4996 )
  #endif
  //---------------------------------------------------------------------------
} // namespace ini
