//-----------------------------------------------------------------------------
/*!
\file IniLineFactory.cpp
\brief Source for LineFactory
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the IniFile module.

IniFile is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General
Public License as published by the Free Software Foundation; either version 3 of
the License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "IniLineFactory.h"
#include <new>
#include "IniSection.h"
#include "IniKey.h"
#include "EuStdFunctions.h"
//-----------------------------------------------------------------------------
#ifdef _DEBUG_MEMORY_SPY
  #include <MilDebugNew.h>
#endif
//-----------------------------------------------------------------------------
namespace ini
{
  //---------------------------------------------------------------------------
  LineFactory::LineFactory()
  {
  }
  //---------------------------------------------------------------------------
  LineFactory::~LineFactory()
  {
  }
  //---------------------------------------------------------------------------
  int LineFactory::getIniLine( Line *& prIniLine, std::string sCharLine ) const
  {
    int         t_iErr = 0;
    LINE_TYPE   t_eIniLineType;
  
    // Was fürn Typ?
    t_eIniLineType = this->getLineType( sCharLine );
    // Richtige Klasse instanziieren
    switch( t_eIniLineType ) 
    {
      #ifdef _DEBUG_MEMORY_SPY
        case IniCOMMENT:
        case IniEMPTY_LINE:
          prIniLine = new Line( sCharLine, t_eIniLineType );
          break;
        case IniVARIABLE:
          prIniLine = new Key( sCharLine, t_eIniLineType );
          break;
        case IniSECTION:
          prIniLine = new Section( sCharLine, t_eIniLineType );
          break;
        case IniUNDEFINED:
          prIniLine = new Line( sCharLine, t_eIniLineType );
          break;
      #else
        case IniCOMMENT:
        case IniEMPTY_LINE:
          prIniLine = new (std::nothrow) Line( sCharLine, t_eIniLineType );
          break;
        case IniVARIABLE:
          prIniLine = new (std::nothrow) Key( sCharLine, t_eIniLineType );
          break;
        case IniSECTION:
          prIniLine = new (std::nothrow) Section( sCharLine, t_eIniLineType );
          break;
        case IniUNDEFINED:
          prIniLine = new (std::nothrow) Line( sCharLine, t_eIniLineType );
          break;
      #endif
    }
    if ( ! prIniLine )
    {
      t_iErr = CREATE;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  void LineFactory::trim( std::string & rsCharLine ) const
  {
    eut::StdStringFunctions::trimString( rsCharLine, 0x20 );
    eut::StdStringFunctions::trimString( rsCharLine, 0x09 );
  }
  //---------------------------------------------------------------------------
  LINE_TYPE LineFactory::getLineType( std::string sCharLine ) const
  {
    this->trim( sCharLine );
    if ( ! sCharLine.empty() )
    {
      if ( this->isComment( sCharLine ) )
      {
        return IniCOMMENT;
      }
      else if ( this->isSection( sCharLine ) )
      {
        return  IniSECTION;
      }
      else if ( this->isKey( sCharLine ) )
      {
        return IniVARIABLE;
      }
      else
      {
        return IniUNDEFINED;
      }
    }
    return IniEMPTY_LINE;
  }
  //---------------------------------------------------------------------------
  bool LineFactory::isComment( std::string sCharLine ) const
  {
    if ( ! sCharLine.empty() )
    {
      if ( sCharLine[0] == '#' || sCharLine[0] == ';' )
      {
        return true;
      }
    }
    return false;
  }
  //---------------------------------------------------------------------------
  bool LineFactory::isSection( std::string sCharLine ) const
  {
    if ( ! sCharLine.empty() )
    {
      if ( sCharLine[0] == '['
           &&
           sCharLine[sCharLine.length() -1] == ']' )
      {
        return true;
      }
    }
    return false;
  }
  //---------------------------------------------------------------------------
  bool LineFactory::isKey( std::string sCharLine ) const
  {
    if ( ! sCharLine.empty() )
    {
      std::size_t   t_Position = sCharLine.find( "=" );

      if ( t_Position != std::string::npos && t_Position != 0 )
      {
        return true;
      }
    }
    return false;
  }
  //---------------------------------------------------------------------------
} // namespace ini
