#ifndef INC_EUBLOB_H
#define INC_EUBLOB_H
//-----------------------------------------------------------------------------
/*!
\file EuBlob.h
\brief
\if english
  Header for Blob
\endif

\if german
  Header für Blob
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTools module.

EuTools is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "EuToolTypes.h"
//#include <new>
//#include <memory.h>
#include <cstddef>
//-----------------------------------------------------------------------------
namespace eut
{
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    This class allows to edit a blob (insert, copy, search etc.).
  \endif

  \if german
    Diese Klasse erlaubt das Bearbeiten eines Blobs (einfügen, kopieren,
    durchsuchen etc.).
  \endif

  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  class __tool_export_dll Blob
  {
    protected:
      //============== Attributes
      //-----------------------------------------------------------------------
      /*!
      \if english
        The blob.
      \endif
    
      \if german
        Das Blob.
      \endif
      */
      char * m_pBlob;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The size of the blob.
      \endif
    
      \if german
        Die Größe des Blobs.
      \endif
      */
      std::size_t m_nSize;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The current position in the blob (starts at 0).
      \endif
    
      \if german
        Die aktuelle Position im Blob (Start bei 0).
      \endif
      */
      std::size_t m_nCurrentPosition;
      //-----------------------------------------------------------------------
      /*!
      \if english
        If a char* is passed, which was not copied or should remain in the
        responsibility of the calling instance, this buffer must not be changed
        or deleted! So here the info is set to false.
      \endif
    
      \if german
        Wenn ein char* übergeben wird, welcher nicht kopiert wurde bzw. in der
        Verantwortung der aufrufenden Instanz verbleiben soll, darf dieser Buffer
        nicht verändert oder gelöscht werden! Also ist hier die Info auf false
        gesetzt.
      \endif
      */
      bool m_bMemResponsible;
      //-----------------------------------------------------------------------

    public:
      //============== Constructors, destructor
      //-----------------------------------------------------------------------
      /*!
      \if english
        Standard constructor.
      \endif
    
      \if german
        Standard-Konstruktor.
      \endif
      */
      Blob();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing.
        \param[in] pData A pointer to an allocated memory.
        \param[in] nDataSize The size of pData.
        \param[in] bCopy If false, pData is not copied, but only #m_pBlob is set
          to pData.
        \attention If in the case of <B>bCopy = false</B> the calling instance
          deletes the buffer before deleting the blob, the method setNULL() must
          be called before further processing to prevent working in freed memory.
        \exception eut::ErrorNException An exception is thrown if an error occurs.
      \endif
    
      \if german
        Konstruktor mit Parameterübergabe.
        \param[in] pData Ein Zeiger auf einen allocierten Speicher.
        \param[in] nDataSize Die Größe von pData.
        \param[in] bCopy Wenn false, wird pData nicht kopiert, sondern nur #m_pBlob
          auf pData gesetzt.
        \attention Wenn im Fall von <B>bCopy = false</B> die aufrufende Instanz den
          Puffer vor dem deleten des Blobs löscht, muss vor dem weiteren bearbeiten
          die Methode setNULL() aufgerufen werden um ein Arbeiten im freigegebenen
          Speicher zu verhindern.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      Blob( char * pData, std::size_t nDataSize, bool bCopy = true );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Copy constructor.
        \param[in] rBlob From this blob the data is copied.
      \endif
    
      \if german
        Copy-Konstruktor.
        \param[in] rBlob Von diesem Blob werden die Daten übernommen.
      \endif
      */
      Blob( const Blob & rBlob );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor, possibly allocated memory is cleaned up.
      \endif
    
      \if german
        Destruktor, ggf. allocierter Speicher wird aufgeräumt.
      \endif
      */
      virtual ~Blob();
      //-----------------------------------------------------------------------

      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        The contents of the attributes of this object are swapped with those of
        the passed object.


        This only works if both are #m_bMemResponsible or both are not
        #m_bMemResponsible.

        \param[in,out] rSwapBlob The blob with which the content is to be swapped.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Es werden die Inhalte der Attribute dieses Objektes mit dem des übergebenen
        Objektes getauscht.

        Das geht nur, wenn beide #m_bMemResponsible bzw. beide nicht
        #m_bMemResponsible sind.

        \param[in,out] rSwapBlob Das Blob mit dem der Inhalt getauscht werden soll.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int swap( Blob & rSwapBlob );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sets the blob.
        \param[in] pData A pointer to an allocated memory.
        \param[in] nDataSize The size of pData.
        \param[in] bCopy If false, pData is not copied, but only #m_pBlob is set
          to pData.
        \attention If in the case of <B>bCopy = false</B> the calling instance
          deletes the buffer before deleting the blob, the method setNULL() must
          be called before further processing to prevent working in freed memory.
        \return A return < 0 indicates an error.
      \endif
    
      \if german
        Setzt das Blob.
        \param[in] pData Ein Zeiger auf einen allocierten Speicher.
        \param[in] nDataSize Die Größe von pData.
        \param[in] bCopy Wenn false, wird pData nicht kopiert, sondern nur #m_pBlob
          auf pData gesetzt.
        \attention Wenn im Fall von <B>bCopy = false</B> die aufrufende Instanz den
          Puffer vor dem deleten des Blobs löscht, muss vor dem weiteren bearbeiten
          die Methode setNULL() aufgerufen werden um ein Arbeiten im freigegebenen
          Speicher zu verhindern.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int setData( char * pData, std::size_t nDataSize, bool bCopy = true );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sets the responsibility over the memory of the blob.

        \param[in] bActiv If true, the blob takes responsibility for the buffer;
        allows editing and deletes the buffer in the destructor. Can be used if the
        calling instance has allocated memory and wants to pass it. However, it is
        then no longer allowed to edit or delete it.
      \endif
    
      \if german
        Setzt die Verantwortung über den Speicher des Blobs.

        \param[in] bActiv Wenn true, übernimmt das Blob die Verantwortung für den
          Puffer; läßt die Bearbeitung zu und deletet den Puffer im Destruktor.
          Kann genutzt werden, wenn die aufrufende Instanz Speicher allociert hat
          und diesen übergeben möchte. Darf diesen dann aber nicht mehr bearbeiten
          oder löschen.
      \endif
      */
      void setMemResponsible( bool bActiv = true );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the responsibility about the memory of the blob.

        \return If true, the blob has responsibility for the buffer; allows editing
        and deletes the buffer in destructor.
      \endif
    
      \if german
        Liefert die Verantwortung über den Speicher des Blobs.

        \return Wenn true, hat das Blob die Verantwortung für den Puffer; läßt die
          Bearbeitung zu und deletet den Puffer im Destruktor.
      \endif
      */
      bool isMemResponsible();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sets #m_pBlob to \ref NULL_PTR "NULL_PTR".

        Frees the memory of #m_pBlob if #m_bMemResponsible = true and set all values
        to the initial size.
      \endif
    
      \if german
        Setzt #m_pBlob auf \ref NULL_PTR "NULL_PTR".

        Gibt den Speicher von #m_pBlob frei wenn #m_bMemResponsible = true und setzt
        alle Werte auf die Anfangsgröße.
      \endif
      */
      void clear();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sets #m_pBlob to \ref NULL_PTR "NULL_PTR" and #m_nSize and #m_nCurrentPosition to 0 without
        removing the #m_pBlob from memory!

        \attention Can be used with caution if this blob gets an uncopied buffer and
        should only be used for iterating and searching, and the buffer should be
        preserved for the calling instance.
      \endif
    
      \if german
        Setzt #m_pBlob auf \ref NULL_PTR "NULL_PTR" und #m_nSize und #m_nCurrentPosition auf 0,
        ohne das #m_pBlob aus dem Speicher zu entfernen!

        \attention Kann mit Vorsicht genutzt werden, wenn dieser Blob einen NICHT
          kopierten Buffer bekommt und nur zum iterieren und suchen genutzt werden
          soll, und der Buffer für die aufrufende Instanz erhalten bleiben soll.
      \endif
      */
      void setNULL();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the blob in prBlob.
        \param[in,out] prBlob The pointer to the memory area of the blob.
        \param[in] bCopy If parameter bCopy is set to true, a copy of the blob is
          returned.
        \return The size of prBlob (number of bytes).
        \attention If parameter bCopy is set to true, this method allocates memory
          to prBlob, which must be released by the calling instance.
        \exception eut::ErrorNException An exception is thrown if no memory could be allocated.
      \endif
    
      \if german
        Liefert in prBlob das Blob.
        \param[in,out] prBlob Der Zeiger auf den Speicherbereich des Blobs.
        \param[in] bCopy Wenn Parameter bCopy auf true gesetzt ist, wird eine Kopie
          des Blobs geliefert.
        \return Die Grösse von prBlob (Anzahl Bytes).
        \attention Wenn Parameter bCopy auf true gesetzt ist, wird von dieser Methode
          für prBlob Speicher allokiert, welcher von der aufrufenden Instanz wieder
          freigegeben werden muß.
        \exception eut::ErrorNException Es wird eine Exception geworfen, wenn kein Speicher
          allociert werden konnte.
      \endif
      */
      virtual std::size_t getData( char *& prBlob, bool bCopy = true );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns a range of the blob as char*.
        \param[in,out] prBlob Pointer to the allocated memory area.
        \param[in] nStartPosition The start position (smallest value 1).
        \param[in] nSize The size of the area (number of bytes).
        \attention This method allocates memory for passed pointers, which must
        be released again by the calling instance.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Liefert einen Bereich des Blobs als char*.
        \param[in,out] prBlob Zeiger auf den alloclierten Speicherbereich.
        \param[in] nStartPosition Die Startposition (kleinster Wert 1).
        \param[in] nSize Die Größe des Bereichs (Anzahl Bytes).
        \attention Es wird von dieser Methode für übergebenen Pointer Speicher
          allokiert, welcher von der aufrufenden Instanz wieder freigegeben werden
          muß.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int getData( char *& prBlob, std::size_t nStartPosition, std::size_t nSize );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns a partial blob of the blob as pointer to new blob.
        \param[in,out] prBlob Pointer to new blob.
        \param[in] nStartPosition The start position (smallest value 1).
        \param[in] nSize The size of the area (number of bytes) which is in the
          delivered new blob.
        \attention This method allocates memory for the passed pointer, which must
          be released by the calling instance.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Liefert einen Teilblob des Blobs als Zeiger auf neuen Blob.
        \param[in,out] prBlob Zeiger auf neues Blob.
        \param[in] nStartPosition Die Startposition (kleinster Wert 1).
        \param[in] nSize Die Größe des Bereichs (Anzahl Bytes) welcher im gelieferten
          neuen Blob ist.
        \attention Es wird von dieser Methode für den übergebenen Zeiger Speicher
          allokiert, welcher von der aufrufenden Instanz wieder freigegeben werden
          muß.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int getBlob( Blob *& prBlob, std::size_t nStartPosition, std::size_t nSize );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns a single char.
        \param[in] nPosition Position Position of the desired char (smallest value 1).
        \attention Make sure that the position pointer is within the range of the
          blob (<B>none</B> Exception).
        \return The char at the overshot position.
      \endif
    
      \if german
        Liefert ein einzelnen char.
        \param[in] nPosition Position des gewünschten Char (kleinster Wert 1).
        \attention Es ist darauf zu achten, daß der Positionszeiger im Range des
          Blobs liegt (<B>keine</B> Exception).
        \return Der char an der übergebnenen Position.
      \endif
      */
      char getChar( std::size_t nPosition );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the blob size.
        \return the blob size.
      \endif
    
      \if german
        Liefert die Blobgröße (Anzahl Bytes).
        \return Die Blobgröße.
      \endif
      */
      std::size_t getSize() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the position in the blob of the passed search mask.
        \param[in,out] nFoundPosition If found, the position in the blob (smallest
          value 1).
        \param[in] pSearchMask The search mask (e.g. char t_szSuchMaske[] = {0x0D, 0x0A};)
        \param[in] nSearchMaskSize Since all bytes can be searched for, the length
          (number of bytes) must be passed.
        \param[in] bSearchAtTheBeginning If true, the search starts at the beginning
          of the blob; if false, the search starts from the current position towards
          the end.
        \return If false, then the search mask was not found.
        \attention m_nCurrentPosition is set to the beginning of the range that
          corresponds to the search mask.
      \endif
    
      \if german
        Liefert die Position im Blob von übergebener Suchmaske.
        \param[in,out] nFoundPosition Wenn gefunden, die Position im Blob (kleinster
          Wert 1).
        \param[in] pSearchMask Die Suchmaske (z.B. char t_szSuchMaske[] = {0x0D, 0x0A};)
        \param[in] nSearchMaskSize Da nach allen Bytes gesucht werden kann, muss die
          Länge (Anzahl Bytes) übergeben werden.
        \param[in] bSearchAtTheBeginning Wenn true, dann wird vom Anfang des Blobs
          gesucht; bei false ab der aktuellen Position in Richtung Ende.
        \return Wenn Rückgabe false, dann Suchmaske nicht gefunden.
        \attention m_nCurrentPosition wird auf den Anfang des Bereichs gesetzt,
          welcher der Suchmaske entspricht.
      \endif
      */
      bool findForward( std::size_t & nFoundPosition,
                        const char * pSearchMask,
                        std::size_t nSearchMaskSize,
                        bool bSearchAtTheBeginning );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the position in the blob of the passed search mask. The search
        starts at the beginning of the blob and moves towards the end.
        \param[in,out] nFoundPosition If found, the position in the blob (smallest
          value 1).
        \param[in] pSearchMask The search mask (e.g. char t_szSuchMaske[] = {0x0D, 0x0A};)
        \param[in] nSearchMaskSize Since all bytes can be searched for, the length
          (number of bytes) must be passed.
        \return If return false, then search mask was not found.

        Used for iterating through the blob in conjunction with findForwardNext (..).

        <B>Code snippets</B>
      \endif
    
      \if german
        Liefert die Position im Blob von übergebener Suchmaske. Gesucht wird vom
        Anfang des Blobs in Richtung Ende.
        \param[in,out] nFoundPosition Wenn gefunden, die Position im Blob (kleinster
          Wert 1).
        \param[in] pSearchMask Die Suchmaske (z.B. char t_szSuchMaske[] = {0x0D, 0x0A};)
        \param[in] nSearchMaskSize Da nach allen Bytes gesucht werden kann, muss
          die Länge (Anzahl Bytes) übergeben werden.
        \return Wenn Rückgabe false, dann wurde Suchmaske nicht gefunden.

        Wird zum Iterieren durch das Blob in Verbindung mit findForwardNext (..) genutzt.

        <B>Codeschnipsel</B>
      \endif

      \code{.cpp}
      std::size_t   t_nEndPosition = 0;
      char          t_szSearchMask[] = {0x0D, 0x0A};
      std::size_t   t_nSearchMaskSize = sizeof(t_szSearchMask);

      ...
      if ( pBlob->findForward( t_nEndPosition, t_szSearchMask, t_nSearchMaskSize ) )
      {
        do
        {
          ... do anything ...
        } while ( pBlob->findForwardNext( t_nEndPosition, t_szSearchMask, t_nSearchMaskSize ) );
      }
      \endcode

      \if english
        \attention m_nCurrentPosition is set to the beginning of the range that
          corresponds to the search mask.
      \endif
    
      \if german
        \attention m_nCurrentPosition wird auf den Anfang des Bereichs gesetzt,
          welcher der Suchmaske entspricht.
      \endif
      */
      bool findForward( std::size_t & nFoundPosition,
                        const char * pSearchMask,
                        std::size_t nSearchMaskSize );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the position in the blob of the passed search mask. The search
        starts from the current Position of the blob towards the end.
        \param[in,out] nFoundPosition If found, the position in the blob (smallest
          value 1).
        \param[in] pSearchMask The search mask (e.g. char t_szSuchMaske[] = {0x0D, 0x0A};)
        \param[in] nSearchMaskSize Since all bytes can be searched for, the Length
          (number of bytes) is transferred.
        \return If return false, then search mask was not found.
        \attention m_nCurrentPosition is set to the beginning of the range, which
          corresponds to the search mask.
      \endif
    
      \if german
        Liefert die Position im Blob von übergebener Suchmaske. Gesucht wird ab der
        aktuellen Position des Blobs in Richtung Ende.
        \param[in,out] nFoundPosition Wenn gefunden, die Position im Blob (kleinster
          Wert 1).
        \param[in] pSearchMask Die Suchmaske (z.B. char t_szSuchMaske[] = {0x0D, 0x0A};)
        \param[in] nSearchMaskSize Da nach allen Bytes gesucht werden kann, muss die
          Länge (Anzahl Bytes) übergeben.
        \return Wenn Rückgabe false, dann wurde Suchmaske nicht gefunden.
        \attention m_nCurrentPosition wird auf den Anfang des Bereichs gesetzt,
          welcher der Suchmaske entspricht.
      \endif
      \sa findForward(std::size_t &, const char *, std::size_t)
      */
      bool findForwardNext( std::size_t & nFoundPosition,
                            const char * pSearchMask,
                            std::size_t nSearchMaskSize );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the position in the blob of the passed search mask.
        \param[in,out] nFoundPosition If found, the position in the blob (smallest
          value 1).
        \param[in] pSearchMask The search mask (e.g. char t_szSuchMaske[] = {0x0D, 0x0A};)
        \param[in] nSearchMaskSize Since all bytes can be searched for, the length
          (number of bytes) must be passed.
        \param[in] bSearchAtTheEnd If true, the search starts at the end of the
          blob; if false, the search starts from the current position towards the
          beginning.
        \return If false, then the search mask was not found.
        \attention m_nCurrentPosition is set to the beginning of the range that
          corresponds to the search mask.
      \endif
    
      \if german
        Liefert die Position im Blob von übergebener Suchmaske.
        \param[in,out] nFoundPosition Wenn gefunden, die Position im Blob (kleinster
          Wert 1).
        \param[in] pSearchMask Die Suchmaske (z.B. char t_szSuchMaske[] = {0x0D, 0x0A};)
        \param[in] nSearchMaskSize Da nach allen Bytes gesucht werden kann, muss die
          Länge (Anzahl Bytes) übergeben.
        \param[in] bSearchAtTheEnd Wenn true, dann wird vom Ende des Blobs
          gesucht; bei false ab der aktuellen Position in Richtung Anfang.
        \return Wenn Rückgabe false, dann Suchmaske nicht gefunden.
        \attention m_nCurrentPosition wird auf den Anfang des Bereichs gesetzt,
          welcher der Suchmaske entspricht.
      \endif
      */
      bool findBackward( std::size_t & nFoundPosition,
                         const char * pSearchMask,
                         std::size_t nSearchMaskSize,
                         bool bSearchAtTheEnd );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the position in the blob of the passed search mask. The search
        starts at the end of the blob and moves towards the beginning.
        \param[in,out] nFoundPosition If found, the position in the blob (smallest
          value 1).
        \param[in] pSearchMask The search mask (e.g. char t_szSuchMaske[] = {0x0D, 0x0A};)
        \param[in] nSearchMaskSize Since all bytes can be searched for, the length
          (number of bytes) must be passed.
        \return If return false, then search mask was not found.

        Used for iterating through the blob in conjunction with findBackwardNext (..).

        <B>Code snippets</B>
      \endif
    
      \if german
        Liefert die Position im Blob von übergebener Suchmaske. Gesucht wird vom
        Ende des Blobs in Richtung Anfang.
        \param[in,out] nFoundPosition Wenn gefunden, die Position im Blob (kleinster
          Wert 1).
        \param[in] pSearchMask Die Suchmaske (z.B. char t_szSuchMaske[] = {0x0D, 0x0A};)
        \param[in] nSearchMaskSize Da nach allen Bytes gesucht werden kann, muss die
          Länge (Anzahl Bytes) übergeben.
        \return Wenn Rückgabe false, dann wurde Suchmaske nicht gefunden.

        Wird zum Iterieren durch das Blob in Verbindung mit findBackwardNext (...) genutzt.

        <B>Codeschnipsel</B>
      \endif

      \code{.cpp}
      std::size_t   t_nEndPosition = 0;
      char          t_szSearchMask[] = {0x0D, 0x0A};
      std::size_t   t_nSearchMaskSize = sizeof(t_szSearchMask);

      ...
      if ( pBlob->findBackward( t_nEndPosition, t_szSearchMask, t_nSearchMaskSize ) )
      {
        do
        {
          ... do anything ...
        } while ( pBlob->findBackwardNext( t_nEndPosition, t_szSearchMask, t_nSearchMaskSize ) );
      }
      \endcode

      \if english
        \attention #m_nCurrentPosition is set to the beginning of the range that
          corresponds to the search mask.
      \endif
    
      \if german
        \attention #m_nCurrentPosition wird auf den Anfang des Bereichs gesetzt,
          welcher der Suchmaske entspricht.
      \endif
      */
      bool findBackward( std::size_t & nFoundPosition,
                         const char * pSearchMask,
                         std::size_t nSearchMaskSize );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the position in the blob of the passed search mask. The search
        starts from the current position of the blob towards the beginning.
        \param[in,out] nFoundPosition If found, the position in the blob (smallest
          value 1).
        \param[in] pSearchMask The search mask (e.g. char t_szSuchMaske[] = {0x0D, 0x0A};)
        \param[in] nSearchMaskSize Since all bytes can be searched for, the Length
          (number of bytes) is transferred.
        \return If return false, then search mask was not found.
        \attention m_nCurrentPosition is set to the beginning of the range, which
          corresponds to the search mask.
      \endif
    
      \if german
        Liefert die Position im Blob von übergebener Suchmaske. Gesucht wird ab der
        aktuellen Position des Blobs in Richtung Anfang.
        \param[in,out] nFoundPosition Wenn gefunden, die Position im Blob (kleinster
          Wert 1).
        \param[in] pSearchMask Die Suchmaske (z.B. char t_szSuchMaske[] = {0x0D, 0x0A};)
        \param[in] nSearchMaskSize Da nach allen Bytes gesucht werden kann, muss die
          Länge (Anzahl Bytes) übergeben.
        \return Wenn Rückgabe false, dann Suchmaske nicht gefunden.
        \attention m_nCurrentPosition wird auf den Anfang des Bereichs gesetzt,
          welcher der Suchmaske entspricht.
      \endif
      */
      bool findBackwardNext( std::size_t & nFoundPosition,
                             const char * pSearchMask,
                             std::size_t nSearchMaskSize );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sets the current position in the blob.

        \param[in] nNewPosition The new position of the current pointer in the
          blob (smallest value 1).
        \return If the position is outside the size of the blob, an error is
          returned.
      \endif
    
      \if german
        Setzt die aktuelle Position im Blob.

        \param[in] nNewPosition Die neue Position des aktuellen Zeigers im Blob
          (kleinster Wert 1).
        \return Wenn die Position ausserhalb der Größe des Blobs ist, wird ein
          Fehler geliefert.
      \endif
      */
      int setCurrentPosition( std::size_t nNewPosition );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the current position of #m_nCurrentPosition in the blob.
        \return The current position (smallest value 1).
      \endif
    
      \if german
        Liefert die aktuelle Position von #m_nCurrentPosition im Blob.
        \return Die aktuelle Position (kleinster Wert 1).
      \endif
      */
      std::size_t getCurrentPosition() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Copies the passed buffer to the blob if #m_bMemResponsible == true.
        \param[in] pBuffer The pointer to the memory to append.
        \param[in] nBufferSize The size (number of bytes) of pBuffer.

        <B>Code snippets</B>
      \endif
    
      \if german
        Kopiert den übergebenen Buffer an das Blobende, wenn #m_bMemResponsible == true.
        \param[in] pBuffer Der Zeiger auf den anzuhängenden Speicher.
        \param[in] nBufferSize Die Größe (Anzahl Bytes) von pBuffer.

        <B>Codeschnipsel</B>
      \endif

      \code{.cpp}
      int         iErr = 0;
      char      * szString = "12345";
      eut::Blob * pBlob = new eut::Blob();

      iErr = pBlob->append( szString, strlen( szString )+1 );
      delete pBlob;
      \endcode

      \if english
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int append( const char * pBuffer, std::size_t nBufferSize );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Copies the passed blob to the blob end if #m_bMemResponsible == true.
        \param[in] pBlob The pointer to the blob to attach.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Kopiert den übergebenen Blob an das Blobende, wenn #m_bMemResponsible == true.
        \param[in] pBlob Der Zeiger auf das anzuhängenden Blob.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int append( const Blob * pBlob );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Copies the passed blob to the blob end if #m_bMemResponsible == true.
        \param[in] rBlob The reference to the blob to be attached.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Kopiert das übergebene Blob an das Blobende, wenn #m_bMemResponsible == true.
        \param[in] rBlob Die Referenz auf das anzuhängenden Blob.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int append( const Blob & rBlob );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Copies the passed buffer into the own blob in passed position.
        \param[in] pBuffer Pointer to memory to be inserted.
        \param[in] nBufferSize The size (number of bytes) of pBuffer.
        \param[in] nBufferPosition The position (smallest value 1) in the blob from
          which to insert.
      \endif
    
      \if german
        Kopiert übergebenen Buffer in das eigene Blob in übergebene Position.
        \param[in] pBuffer Zeiger auf einzufügenden Speicher.
        \param[in] nBufferSize Die Größe (Anzahl Bytes) von pBuffer.
        \param[in] nBufferPosition Die Position (kleinster Wert 1) im Blob, ab dem
          eingefügt werden soll.

        <B>Codeschnipsel</B>
      \endif

      \code{.cpp}
      int         iErr = 0;
      char      * szString = "12345";
      eut::Blob * pBlob = new eut::Blob();

      // szString ans Ende von pBlob anhängen
      iErr = pBlob->insert( szString, strlen( szString )+1, pBlob->getSize()+1 );
      ...
      delete pBlob;
      \endcode

      \if english
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int insert( const char * pBuffer,
                  std::size_t nBufferSize,
                  std::size_t nBufferPosition );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Copies the transferred blob into its own blob in the transferred position.
        \param[in] pBlob The pointer to the blob to be inserted.
        \param[in] nBufferPosition The position (smallest value 1) in the blob from
          which to insert.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Kopiert übergebenes Blob in das eigene Blob in übergebene Position.
        \param[in] pBlob Der Zeiger auf das einzufügendes Blob.
        \param[in] nBufferPosition Die Position (kleinster Wert 1) im Blob, ab dem
          eingefügt werden soll.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int insert( const Blob * pBlob, std::size_t nBufferPosition );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Copies the transferred blob into its own blob in the transferred position.
        \param[in] rBlob The reference to the blob to be inserted.
        \param[in] nBufferPosition The position (smallest value 1) in the blob from
          which to insert.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Kopiert übergebenes Blob in das eigene Blob in übergebene Position.
        \param[in] rBlob Die Referenz auf das einzufügende Blob.
        \param[in] nBufferPosition Die Position (kleinster Wert 1) im Blob, ab dem
          eingefügt werden soll.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int insert( const Blob & rBlob, std::size_t nBufferPosition );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Removes transferred number of bytes in own blob starting from the transferred
        position (including this one).
        \param[in] nNumberRemoveBytes The number of bytes to remove.
        \param[in] nBufferPosition The position (smallest value 1) in the blob from
          which to remove.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Entfernt übergebene Anzahl von Bytes im eigenen Blob ab der übergebenen
        Position (inkl. dieser).
        \param[in] nNumberRemoveBytes Die Anzahl der zu entfernenden Bytes.
        \param[in] nBufferPosition Die Position (kleinster Wert 1) im Blob, ab dem
          entfernt werden soll.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int remove( std::size_t nNumberRemoveBytes, std::size_t nBufferPosition );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Searches and removes first passed content pOldContent from the blob once
        from the beginning and inserts the content pNewContent instead.
        \param[in] pOldContent The content to be exchanged.
        \param[in] pOldContentSize The length of the content to be exchanged.
        \param[in] pNewContent The new content to be inserted.
        \param[in] pNewContentSize The length of the content to be inserted.
        \return If 1, the content was exchanged and if 0, the content to be
          exchanged was not found. <br>
          A return value < 0 indicates an error.
      \endif
    
      \if german
        Sucht und entfernt ersten übergebenen Inhalt pOldContent ab Anfang
        einmalig aus dem Blob und fügt statt dessen den Inhalt pNewContent ein.
        \param[in] pOldContent Die auszutauschende Inhalt.
        \param[in] pOldContentSize Die Länge des auszutauschenden Inhalts.
        \param[in] pNewContent Die neu einzufügende Inhalt.
        \param[in] pNewContentSize Die Länge des einzufügenden Inhalts.
        \return Wenn 1, ist der Inhalt ausgetauscht und wenn 0, wurde der
          auszutauschende Inhalt nicht gefunden. <br>
          Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int replace(  const char * pOldContent,
                    std::size_t pOldContentSize,
                    const char * pNewContent,
                    std::size_t pNewContentSize );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Searches and removes all content equal to the first passed content pOldContent
        from the blob and inserts the content from pNewContent instead.
        \param[in] pOldContent The content to be exchanged.
        \param[in] pOldContentSize The length of the content to be exchanged.
        \param[in] pNewContent The new content to be inserted.
        \param[in] pNewContentSize The length of the content to be inserted.
        \return Is the number of times the content was exchanged. <br>
          A return value < 0 indicates an error.
        \attention If an error occurs, the state of the blob is undefined; contents
          may already have been swapped.
      \endif
    
      \if german
        Sucht und entfernt alle Inhalte gleich dem ersten übergebenen Inhalt pOldContent
        aus dem Blob und fügt statt dessen jeweils den Inhalt aus pNewContent ein.
        \param[in] pOldContent Die auszutauschende Inhalt.
        \param[in] pOldContentSize Die Länge des auszutauschenden Inhalts.
        \param[in] pNewContent Die neu einzufügende Inhalt.
        \param[in] pNewContentSize Die Länge des einzufügenden Inhalts.
        \return Ist die Anzahl, wie oft der Inhalt ausgetauscht wurde. <br>
          Eine Rückgabe < 0 zeigt einen Fehler an.
        \attention Wenn ein Fehler auftritt, ist der Zustand des Blobs undefiniert;
          es können schon Inhalte getauscht worden sein.
      \endif
      */
      int replaceAll( const char * pOldContent,
                      std::size_t pOldContentSize,
                      const char * pNewContent,
                      std::size_t pNewContentSize );
      //-----------------------------------------------------------------------

      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        Assignment operator which expects an Blob and appends it to its data.
        \param[in] rBlob From this object the data is taken over.
        \exception eut::ErrorNException An exception is thrown if an error occurs.
      \endif
    
      \if german
        Zuweisungsoperator welcher ein Blob erwartet und diesen an seine Daten
        anhängt.
        \param[in] rBlob Von diesem Objekt werden die Daten übernommen.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      \sa Blob::append( Blob & rBlob )
      */
      Blob & operator+= ( const Blob & rBlob );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Assignment operator which expects an Blob.
        \param[in] rBlob The data is transferred from this object.
      \endif
    
      \if german
        Zuweisungsoperator welcher ein Blob erwartet.
        \param[in] rBlob Von diesem Objekt werden die Daten übernommen.
      \endif
      */
      Blob & operator= ( const Blob & rBlob );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Assignment operator which expects an integer.
        \param[in] iNULL No matter what is passed; the blob is removed from memory
          and all values are set to 0.

        In any case the Blob has an "invalid" value (operator int() returns 0)!
      \endif
    
      \if german
        Zuweisungsoperator welcher ein integer erwartet.
        \param[in] iNULL Egal was übergeben wird; es werden das Blob aus dem Speicher
          entfernt und alle Werte auf 0 gesetzt.

        Auf jedenfall hat das Blob einen "ungültigen" Wert (operator int() liefert 0)!
      \endif
      */
      Blob & operator= ( int iNULL );
      //-----------------------------------------------------------------------
      /**@{*/
      /*!
      \if english
        Comparison operator.
      \endif
    
      \if german
        Vergleichsoperator.
      \endif
      */
      int operator== ( const Blob & rBlob ) const;
      int operator!= ( const Blob & rBlob ) const;
      bool operator> ( const Blob & rBlob );
      bool operator< ( const Blob & rBlob );
      bool operator>= ( const Blob & rBlob );
      bool operator<= ( const Blob & rBlob );
      /**@}*/
      //-----------------------------------------------------------------------
      /*!
      \if english
        If the size is not set then the return value is 0.

        <B>code snippets</B>

        \code{.cpp}
        eut::Blob myBlob();

        if ( myBlob > 0 ) cout << "Blob has content!" << endl;
        else cout << "Blob has NO content!" << endl;
        \endcode

        Output:
        \code
        Blob has NO content!
        \endcode
      \endif
    
      \if german
        Wenn die Größe nicht gesetzt dann ist der Rückgabewert gleich 0.

        <B>Codeschnipsel</B>

        \code{.cpp}
        eut::Blob myBlob();

        if ( myBlob > 0 ) cout << "Blob hat Inhalt!" << endl;
        else cout << "Blob hat KEINEN Inhalt!" << endl;
        \endcode

        Ausgabe:
        \code
        Blob hat KEINEN Inhalt!
        \endcode
      \endif
       */
      operator int() const;
      //-----------------------------------------------------------------------
      friend Blob operator+ ( Blob lT, const Blob & rrT )
      {
        lT += rrT;
        return lT;
      };
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
} // namespace eut
#endif
