#ifndef INC_PTLAVDVLIST_H
#define INC_PTLAVDVLIST_H
//-----------------------------------------------------------------------------
/*!
\file PtlAVDVList.h
\brief
\if english
  Header for AVDVList
\endif

\if german
  Header für AVDVList
\endif
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the PtlContainer module.

PtlContainer is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "PtlListTypes.h"
#include "PtlDVElement.h"
#include <new>
#include <stddef.h>
//-----------------------------------------------------------------------------
namespace ptl
{
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Base class for void-pointer lists.
    \details A AVDVList consists of a set of DVElements to which pointers
      to the objects to be managed are passed.

    The first and last can be queried and deleted.

    An element can be appended at the beginning or the end. With parameter
    #UNIQUE will include the list of data objects only once.

    This class is the base class for VDQueue and VDVList.
  \endif

  \if german
    Basisklasse für void-pointer Listen.
    \details  Eine AVDVList besteht aus einer Menge von DVElement, welchen
      Pointer auf die zu verwaltenden Objekte übergeben werden.

    Es können das erste und letzte erfragt sowie gelöscht werden.

    Ein Element kann am Anfang oder am Ende angehängt werden. Mit Parameter
    #UNIQUE wird die Liste Datenobjekte nur einmalig aufnehmen.

    Diese Klasse ist die Basisklasse für VDQueue und VDVList.
  \endif

  \copyright © 1995 Helmut Jakoby
  \author Helmut Jakoby
  */
  class AVDVList
  {
    protected:
      //============== Attribute ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        For DVLists, whether they allow data objects more than once or only once,
        or whether they are the owner (responsible, or destroy also the data objects
        in the destructor) of these data objects.
      \endif
    
      \if german
        Für DVListen, ob sie Datenobjekte mehr- oder nur einmalig zulassen oder
        Besitzer (verantwortlich, bzw. zerstören auch die Datenobjekte im Destruktor)
        dieser Datenobjekte sind.
      \endif
      */
      ListMode m_eListMode;
      //-----------------------------------------------------------------------
      /*!
      \if english
        An error code is stored here (e.g. if something goes wrong during instantiation).
      \endif
    
      \if german
        Hier ist ein Fehlercode abgelegt (z.B. wenn was beim instanziieren schief
        geht).
      \endif
      */
      int m_iErrorCode;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Pointer to the first element in the list. If List is empty, this attribute
        equals \ref NULL_PTR "NULL_PTR".
      \endif
    
      \if german
        Zeiger auf das erste Element in Liste. Wenn Liste leer, ist dieses Attribut
        gleich \ref NULL_PTR "NULL_PTR".
      \endif
      */
      DVElement * m_pFirstDVElement;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Pointer to the last element in the list. If List is empty, this attribute
        equals \ref NULL_PTR "NULL_PTR".
      \endif
    
      \if german
        Zeiger auf das letzte Element in Liste. Wenn Liste leer, ist dieses Attribut
        gleich \ref NULL_PTR "NULL_PTR".
      \endif
      */
      DVElement* m_pLastDVElement;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Number of elements contained in the list.
      \endif
    
      \if german
        Anzahl der in Liste enthaltenen Elemente.
      \endif
      */
      std::size_t m_nSize;
      //-----------------------------------------------------------------------

    public:
      //============== Konstruktoren ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with optional parameter passing.
        The list is empty at first.You can use the '#UNIQUE' parameter to create a
        list that includes data objects only once.
        \param[in] eMode The list mode
      \endif
    
      \if german
        Konstruktor mit optionaler Parameterübergabe.
        Die Liste ist erst einmal leer. Es kann mit Parameter '#UNIQUE' eine Liste
        erstellt werden, welche Datenobjekte nur einmal aufnimmt.
        \param[in] eMode  Der Listenmodus
      \endif
      */
      AVDVList( ListMode eMode = DEFAULT );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor, emptying list.
      \endif
    
      \if german
        Destruktor, Liste wird geleert.
      \endif
      */
      virtual ~AVDVList();
      //-----------------------------------------------------------------------

      //============== Copy-Konstruktoren ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Copy constructor.
        \param[in] rList The list to be copied.
        \exception eut::ErrorNException If an error occurs, a exception is thrown.
      \endif
    
      \if german
        Copy-Konstruktor.
        \param[in] rList Die zu kopierende Liste.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      AVDVList( const AVDVList& rList );
      //-----------------------------------------------------------------------

      //============== Methoden ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns a pointer to the data from passed eElementPos.
        \param[in] eElementPos Which data should be delivered; that of the first
        or the last element. Possible values are ptl::FIRST_POS and ptl::LAST_POS.
        \return The return value is the pointer to the data, otherwise
          \ref NULL_PTR "NULL_PTR".

        <B>Example 1:</B>
        \code{.cpp}
        ptl::AVDVList * myVDVList = new ptl::AVDVList();
        DataObject    * pDataObject;
        ...
        if ( pDataObject = static_cast<DataObject*>( myVDVList->get(ptl::FIRST_POS) ) )
        {
          pDataObject->MachWas();
        }
        \endcode
        The DataObject from the first PtlDV element from the list is delivered if
        it exists, otherwise \ref NULL_PTR "NULL_PTR".

        <B>Example 2:</B>
        \code{.cpp}
        ptl::AVDVList * myVDVList = new ptl::AVDVList();
        DataObject    * pDataObject;
        ...
        if ( pDataObject = static_cast<DataObject*>( myVDVList->get(ptl::LAST_POS) ) )
        {
          pDataObject->MachWas();
        }
        \endcode
        The DataObject from the last PtlDV element from the list is delivered if
        it exists, otherwise \ref NULL_PTR "NULL_PTR".
      \endif
    
      \if german
        Liefert einen Zeiger auf die Daten aus übergebener eElementPos.
        \param[in] eElementPos Welche Daten sollen geliefert werden; die des ersten
        oder des letzten Elements. Mögliche Werte sind ptl::FIRST_POS und ptl::LAST_POS.
        \return Der Rückgabewert ist der Zeiger auf die Daten, sonst
          \ref NULL_PTR "NULL_PTR".

        <B>Beispiel 1:</B>
        \code{.cpp}
        ptl::AVDVList * myVDVList = new ptl::AVDVList();
        DataObject    * pDataObject;
        ...
        if ( pDataObject = static_cast<DataObject*>( myVDVList->get(ptl::FIRST_POS) ) )
        {
          pDataObject->MachWas();
        }
        \endcode
        Es wird das DataObject aus dem ersten DVElement aus der Liste geliefert,
        wenn dieses vorhanden ist, ansonsten \ref NULL_PTR "NULL_PTR".

        <B>Beispiel 2:</B>
        \code{.cpp}
        ptl::AVDVList * myVDVList = new ptl::AVDVList();
        DataObject    * pDataObject;
        ...
        if ( pDataObject = static_cast<DataObject*>( myVDVList->get(ptl::LAST_POS) ) )
        {
          pDataObject->MachWas();
        }
        \endcode
        Es wird das DataObject aus dem letzten DVElement aus der Liste geliefert,
        wenn dieses vorhanden ist, ansonsten \ref NULL_PTR "NULL_PTR".
      \endif

      \sa #ElementPosition, #m_pFirstDVElement und #m_pLastDVElement
      */
      virtual void * get( ElementPosition eElementPos );
      //-----------------------------------------------------------------------
      /*!
      \if english
        This method inserts data objects into the list.

        If the list was instantiated with parameter '#UNIQUE' and the data object
        is already in the list, it is not inserted.
        \param[in] pDataObject Is a pointer to the data object. If this parameter
          is \ref NULL_PTR "NULL_PTR", nothing is inserted.
        \param[in] eWhere Shows where the data object is inserted in the list is
          to be. Possible parameters are #InsertHEAD and #InsertLAST.
        \param[in] eBeforeBehind Specifies whether the new element is to be placed
          before or after the, element specified in the eWhere parameter. Possible
          Parameters are #BEFORE and #BEHIND.
        \return A return value < 0 indicates an error. The following errors are
          possible: #PASSED_NO_OBJECT, #INSTANTIATE_DVELEMENT
          and #OBJECT_ALREADY_AVAILABLE (only at #UNIQUE).

        <B>Example 1:</B>
        \code{.cpp}
        int             iErr = 0;
        ptl::AVDVList * myVDVList = new ptl::AVDVList();
        DataObject    * pDataObject = new DataObject();

        iErr = myVDVList->insert(pDataObject, InsertLAST, BEHIND);
        \endcode
        The DataObject is positioned at the end of the list after the last DVElement (#m_pLastDVElement).

        <B>Example 2:</B>
        \code{.cpp}
        iErr = myVDVList->insert(pDataObject, InsertLAST, BEFORE);
        \endcode
        The DataObject is positioned at the end of the list before the last DVElement (#m_pLastDVElement).

        <B>Example 3:</B>
        \code{.cpp}
        iErr = myVDVList->insert(pDataObject, InsertHEAD, BEHIND);
        \endcode
        The DataObject is positioned at the beginning of the list after the first DVElement (#m_pFirstDVElement).

        <B>Example 4:</B>
        \code{.cpp}
        iErr = myVDVList->insert(pDataObject, InsertHEAD, BEFORE);
        \endcode
        The DataObject is positioned at the beginning of the list before the first DVElement (#m_pFirstDVElement).
      \endif
    
      \if german
        Diese Methode fügt Datenobjekt in die Liste ein.

        Sollte die Liste mit Parameter '#UNIQUE' instanziiert worden sein und das
        Datenobjekt schon in der Liste sein, wird es nicht eingefügt.
        \param[in] pDataObject Ist Zeiger auf das Datenobjekt. Wenn dieser
        Parameter \ref NULL_PTR "NULL_PTR" ist, wird nichts eingefügt.
        \param[in] eWhere Zeigt an, wohin das Datenobjekt in die Liste eingefügt
        werden soll. Mögliche Parameter sind #InsertHEAD und #InsertLAST.
        \param[in] eBeforeBehind Gibt an, ob das neue Element vor oder hinter das,
        im Parameter eWhere angegebene Element, gesetzt wird. Mögliche Parameter
        sind #BEFORE und #BEHIND.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
        Folgende Fehler sind möglich: #PASSED_NO_OBJECT,
        #INSTANTIATE_DVELEMENT und #OBJECT_ALREADY_AVAILABLE
        (nur bei #UNIQUE).

        <B>Beispiel 1:</B>
        \code{.cpp}
        int             iErr = 0;
        ptl::AVDVList * myVDVList = new ptl::AVDVList();
        DataObject    * pDataObject = new DataObject();

        iErr = myVDVList->insert(pDataObject, InsertLAST, BEHIND);
        \endcode
        Es wird das DataObject an das Ende der Liste hinter dem letzten DVElement (#m_pLastDVElement) positioniert.

        <B>Beispiel 2:</B>
        \code{.cpp}
        iErr = myVDVList->insert(pDataObject, InsertLAST, BEFORE);
        \endcode
        Es wird das DataObject an das Ende der Liste vor dem letzten DVElement (#m_pLastDVElement) positioniert.

        <B>Beispiel 3:</B>
        \code{.cpp}
        iErr = myVDVList->insert(pDataObject, InsertHEAD, BEHIND);
        \endcode
        Es wird das DataObject an den Anfang der Liste hinter dem ersten DVElement (#m_pFirstDVElement) positioniert.

        <B>Beispiel 4:</B>
        \code{.cpp}
        iErr = myVDVList->insert(pDataObject, InsertHEAD, BEFORE);
        \endcode
        Es wird das DataObject an den Anfang der Liste vor dem ersten DVElement (#m_pFirstDVElement) positioniert.
      \endif
      */
      virtual int insert( void* pDataObject,
                          InsertPosition eWhere = InsertLAST,
                          VH eBeforeBehind = BEHIND );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the number of elements contained in the list.
        \return The number of elements

        <B>Example:</B>
      \endif
    
      \if german
        Liefert die Anzahl der in Liste enthaltenen Elemente.
        \return Die Anzahl der Elemente

        <B>Beispiel:</B>
      \endif
      \code{.cpp}
      ptl::AVDVList myList;
      int iSize = myListe.getSize()
      \endcode
      */
      std::size_t getSize();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the number of data objects that are equal to the passed pointer of the data object.
        \param[in] pDataObject The pointer to the object
        \return The frequency of occurrence of pDataObject

        <B>Example:</B>
      \endif
    
      \if german
        Liefert die Anzahl der Datenobjekte zurück, welche gleich des übergebenen
        Zeigers des Datenobjekts sind.
        \param[in] pDataObject Der Zeiger auf das Objekt
        \return Die Häufigkeit des Vorkommens von pDataObject

        <B>Beispiel:</B>
      \endif
      \code{.cpp}
      ptl::AVDVList  myList;
      DataObject   * pDataObject = new DataObject();
      ...
      std::size_t nObjCopunter = myList.getNumberDataObjects(pDataObject);
      \endcode
      */
      std::size_t getNumberDataObjects( void* pDataObject );
      //-----------------------------------------------------------------------
      /*!
      \if english
        All DVElements are removed from the list and from memory.
      \endif
    
      \if german
        Es werden alle DVElement aus der Liste und aus dem Speicher entfernt.
      \endif
      */
      virtual void removeAll();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the last error code.

        Should be called after insert methods to check whether errors have occurred.

        If the error #INSTANTIATE_DVELEMENT occurred, the instance
        should be deleted if no other error handling is implemented in subclasses.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Liefert den letzten Fehlercode.

        Sollte nach Einfügemethoden aufgerufen werden, um zu überprüfen, ob Fehler
        aufgetreten sind.

        Ist der Fehler #INSTANTIATE_DVELEMENT aufgetreten, sollte die
        Instanz gelöscht werden, wenn keine andere Fehlerbehandlung in Unterklassen
        implementiert ist.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
      \endif
      \sa ERRORS
      */
      int getErrorCode();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the List mode.
        \return The ListMode ptl::RESPONSIBLE is only possible for template
        lists that use this class. A combination of the values can also be supplied.
        \sa ListMode
        <B>Example:</B>
      \endif
    
      \if german
        Liefert den ListModus.
        \return Der ListMode ptl::RESPONSIBLE ist nur bei Templatelisten
        möglich, welche diese Klasse nutzen. Es kann auch eine Kombination der
        Werte geliefert werden.
        \sa ListMode
        <B>Beispiel:</B>
      \endif
      \code{.cpp}
      ptl::AVDVList  myList;
      ...
      // if myList with UNIQUE-Bit...
      if (myList.getListMode() & UNIQUE)
      {
        // ...do something...
      }
      \endcode
      */
      ListMode getListMode();
      //-----------------------------------------------------------------------

    protected:
      //+ ============ HilfsMethoden ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Copies all list elements of parameter rList to this list.

        Is used by #operator=.

         Data objects from subclasses can be adopted, just as the subclasses can
         transfer data objects among themselves.

         The data objects are not duplicated!

         \param[in] rList The list from which the elements are taken.
         \return A return value <0 indicates an error.

      \endif
    
      \if german
        Kopiert alle Listenelemente von Parameter rList zu dieser Liste hinzu.

        Wird von #operator= genutzt.

        Es können Datenobjekte aus Unterklassen übernommen werden, wie auch die
        Unterklassen untereinander Datenobjekte übertragen können.

        Es werden nicht die Datenobjekte dupliziert!

        \param[in] rList Die Liste, aus der die Elemente übernommen werden.
        \return Ein Rückgabewerte < 0 zeigt einen Fehler an.
      \endif
      */
      virtual int _addElementFrom( const AVDVList & rList );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Inserts a pDataObject before or after the passed DVElement and returns
        returns the new PtlDV element.

        If the list was instantiated with Prameter #UNIQUE, the data object
        is only inserted if it is not yet in the list.

        The attributes #m_pFirstDVElement, #m_pLastDVElement and #m_nSize are updated.

        It should be queried via getErrorCode() to check the error code.

        \param[in] pDataObject Pointer to the data object. If this parameter is
          \ref NULL_PTR "NULL_PTR", no new DVElement is inserted.

        \param[in] pReferenceDVElement The reference element where the new DVElement
          is inserted before or appended.

        \param[in] eBeforeBehind Specifies whether new DVElement is appended to
          reference element is inserted (#BEFORE) or appended (#BEHIND).

        \return The DVElement with reference to the passed pDataObject
      \endif
    
      \if german
        Fügt ein pDataObject vor oder nach dem übergebenen DVElement ein und gibt
        das neue DVElement retour.

        Wenn die Liste #UNIQUE ist, wird das DatenObjekt nur eingefügt, wenn es
        noch nicht in der Liste ist.

        Attribute #m_pFirstDVElement, #m_pLastDVElement und #m_nSize werden aktuallisiert.

        Es sollte zur Überprüfung der ErrorCode über getErrorCode() abgefragt werden.

        \param[in] pDataObject Zeiger auf das Datenobjekt. Wenn dieser Parameter
          \ref NULL_PTR "NULL_PTR" ist, wird kein neues DVElement eingefügt.

        \param[in] pReferenceDVElement Das Referenz-Element, wo das neue DVElement
          vor oder angehängt wird.

        \param[in] eBeforeBehind Giebt an, ob neues DVElement an ReferenzDVElement
          vor- (#BEFORE) oder angehängt (#BEHIND) wird.

        \return Das DVElement mit Referenz auf übergebenes pDataObject
      \endif
      */
      DVElement * insertSrc(  void* pDataObject,
                              DVElement* pReferenceDVElement,
                              VH eBeforeBehind );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Removes passed DVElement from the list
        \param[in] pDeadDVElement The DVElement to be removed
        \return If a subsequent DVElement is present, it is returned, otherwise
        the previous DVElement. If the passed DVElement was the last in the
        list, \ref NULL_PTR "NULL_PTR" is returned.
      \endif
    
      \if german
        Entfernt übergebenes DVElement aus der Liste
        \param[in] pDeadDVElement Das zu entfernende DVElement
        \return Wenn ein nachfolgendes DVElement vorhanden ist, wird dieser retour
        geliefert, ansonsten das vorherige DVElement. Wenn übergebenes DVElement
        das letzte in Liste war, wird \ref NULL_PTR "NULL_PTR" retour geliefert.
      \endif
      */
      DVElement * removeSrc( DVElement * pDeadDVElement );
      //-----------------------------------------------------------------------

    public:
      //============== Operatoren ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Assignment operator=

        During an assignment, all own DVElements are destroyed and then all
        data objects from the assigning VDVList are transferred to new own
        DVElements.
        \param[in] rList The list to be assigned.

        It should be called after call getErrorCode().
      \endif
    
      \if german
        Zuweisungsoperator=

        Bei einer Zuweisung werden alle DVElement zerstört und dann alle Datenobjekte
        aus der zuweisenden VDVList in neue eigene DVElement übernommen.
        \param[in] rList Die zuzuweisende Liste.

        Es sollte nach Aufruf getErrorCode() aufgerufen werden.
      \endif
      */
      AVDVList & operator= ( const AVDVList & rList );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
} // namespace ptl
#endif
