#ifndef INC_PTLVDVLIST_H
#define INC_PTLVDVLIST_H
//-----------------------------------------------------------------------------
/*!
\file PtlVDVList.h
\brief
\if english
  Header for VDVList
\endif

\if german
  Header für VDVList
\endif
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the PtlContainer module.

PtlContainer is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "PtlListTypes.h"
#include "PtlAVDVList.h"
#include "PtlDVElement.h"
//-----------------------------------------------------------------------------
namespace ptl
{
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    A void-pointer list.
    \details A VDVList consists of list elements of type DVElement, which
    pointers to the objects to be managed. The first, last and current element can
    be queried. In addition, a subsequent one can be requested, or previous element
    (with respect to the current element) can be selected.

    An element can be inserted before or after the first, last and current element
    will be. This can be restricted by setting a #UNIQUE list in the following way
    that objects may only appear once in the list.

    \note This class works with void pointers and is therefore not type-safe. It it
    is recommended to use the template class TDVList.
  \endif

  \if german
    Eine Void-Pointer Listen.
    \details  Eine VDVList besteht aus Listen-Elementen des Typs DVElement,
    welchen Pointer auf die zu verwaltenden Objekte haben. Es können das erste, letzte
    und aktuelle Element erfragt werden. Zusätzlich kann nach einem nachfolgenden,
    bzw. vorherigen Element (bezüglich dem aktuellen Element) selektiert werden.

    Ein Element kann vor oder hinter dem ersten, letzten und aktuellen Element eingefügt
    werden. Dieses kann durch Einstellung einer #UNIQUE Liste in der Weise eingeschränkt
    werden, daß Objekte nur einmalig in der Liste vorkommen dürfen.

    \note Diese Klasse arbeitet mit Void-Pointern und ist damit nicht typsicher. Es
    wird empfohlen die Template-Klasse TDVList zu nutzen.
  \endif

  \copyright © 1995 Helmut Jakoby
  \author Helmut Jakoby
  */
  class VDVList : public AVDVList
  {
    protected:
      //============== Attribute ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        The reference to the current element in the list. If the list is empty,
        the current element is equal to \ref NULL_PTR "NULL_PTR".
      \endif
    
      \if german
        Die Referenz auf das aktuelle Element in der Liste. Wenn die Liste leer ist,
        ist das aktuelle Element gleich \ref NULL_PTR "NULL_PTR".
      \endif
      */
      DVElement * m_pCurrentDVElement;
      //-----------------------------------------------------------------------
      /*!
      \if english
        This attribute is a cache of the current element. It is set in the
        currentElementCache (...) method.
      \endif
    
      \if german
        Dieses Attribut ist ein Zwischenspeicher des aktuellen Elements. Wird in
        der Methode currentElementCache (...) gesetzt.
      \endif
      */
      DVElement * m_pTempCurrentDVElement;
      //-----------------------------------------------------------------------

    public:
      //============== Konstruktoren ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        constructor with optional parameter passing.
        The list is empty at first. You can use the parameter #UNIQUE to create a
        list that only takes a data object once.
        \param[in] eMode The list mode
      \endif
    
      \if german
        Konstruktor mit optionaler Parameterübergabe.
        Die Liste ist erst einmal leer. Es kann mit dem Parameter #UNIQUE eine Liste
        erstellt werden, welche ein Datenobjekt nur einmal aufnimmt.
        \param[in] eMode Der Listenmodus
      \endif
      */
      VDVList( ListMode eMode = DEFAULT );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor.
      \endif
    
      \if german
        Destruktor.
      \endif
      */
      virtual ~VDVList ();
      //-----------------------------------------------------------------------

      //============== Copy-Konstruktoren ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Copy constructor.
        \param[in] rList The VDVList to be copied.
        \exception eut::ErrorNException If an error occurs, a exception is thrown.
      \endif
    
      \if german
        Copy-Konstruktor.
        \param[in] rList Die zu kopierende VDVList.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      VDVList( const VDVList& rList );
      //-----------------------------------------------------------------------

      //+ ============ Methoden ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Copies all list elements of type DVElement from rList to this list.
        Also used by VDVList::operator=.
        \sa AVDVList::_addElementFrom(...)
        \param[in] rList The list from which the objects are taken.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Kopiert alle Listenelemente vom Typ DVElement von rList zu dieser Liste
        hinzu. Wird auch von VDVList::operator= genutzt.
        \sa AVDVList::_addElementFrom(...)
        \param[in] rList Die Liste, aus der die Objekte übernommen werden.
        \return Ein Rückgabewerte < 0 zeigt einen Fehler an.
      \endif
      */
      virtual int addElementFrom( const VDVList & rList );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns a void pointer to the data from the position specified in the parameter.

        If an element was delivered, it becomes the current element, otherwise the
        current element remains what it was.
        \param[in] eElementPos Which data should be delivered? Possible values are
        #FIRST_POS, #LAST_POS, #CURRENT_POS, #NEXT_POS or #PREVIOUS_POS.
        \return The return value is the pointer to the data, otherwise
          \ref NULL_PTR "NULL_PTR".
      \endif
    
      \if german
        Liefert einen rohen Zeiger auf die Daten aus der im Parameter angegebenen
        Position.

        Wenn ein Element geliefert wurde, wird dieses zum aktuellen Element, ansonsten
        bleibt das aktuelle Element das was es war.
        \param[in] eElementPos Welche Daten sollen geliefert werden? Mögliche Werte
        sind #FIRST_POS, #LAST_POS, #CURRENT_POS, #NEXT_POS oder #PREVIOUS_POS.
        \return Der Rückgabewert ist der Zeiger auf die Daten, ansonsten
          \ref NULL_PTR "NULL_PTR".
      \endif
      */
      virtual void * get( ElementPosition eElementPos = FIRST_POS ) override;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns a void pointer to an object in prRetVal from the list with respect
        to the transfer parameters.

        You can use this method to iterate through the list.
        \param[in,out] prRetVal In the passed pointer a found object delivered.
        \param[in] nPosition The position relative to the search mode.
        \param[in] eMode The search mode.
        \return A return < 0 indicates an error. A return == #RANGE
          indicates the end of the list.

        <B>Parameter examples:</B>
        \li lPosition = 0; eMode = #START -> Delivers first element from list
        \li lPosition = 0; eMode = #END -> Delivers last element from list
        \li lPosition = 0; eMode = #CURRENT -> Delivers current element from list
        \li lPosition = 1; eMode = #CURRENT -> Delivers next element from list
        \li lPosition = 2; eMode = #CURRENT -> Delivers the next but one element from the list
        \li lPosition = -1; eMode = #CURRENT -> Returns previous element from list
      \endif
    
      \if german
        Liefert einen rohen Zeiger auf ein Objekt in prRetVal aus der Liste in Bezug
        auf die Übergabe-Parameter.

        Mit dieser Methode kann durch die Liste iteriert werden.
        \param[in,out] prRetVal Im übergebenen Zeiger wird ein gefundenes Objekt
          geliefert.
        \param[in] nPosition Die Position rellativ zum Suchmodus.
        \param[in] eMode Der Suchmodus.
        \return Eine Rückgabe < 0 zeigt einen Fehler an. Eine Rückgabe == #RANGE
          zeigt das Ende der Liste an.

        <B>Parameterbeispiele:</B>
        \li lPosition =  0; eMode = #START -> Liefert erstes Element aus Liste
        \li lPosition =  0; eMode = #END -> Liefert letztes Element aus Liste
        \li lPosition =  0; eMode = #CURRENT -> Liefert aktuelles Element aus Liste
        \li lPosition =  1; eMode = #CURRENT -> Liefert nächstes Element aus Liste
        \li lPosition =  2; eMode = #CURRENT -> Liefert übernächstes Element aus Liste
        \li lPosition =  -1; eMode = #CURRENT -> Liefert vorgeriges Element aus Liste
      \endif
      */
      int get( void *& prRetVal, std::size_t nPosition, SeekMode eMode );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns a void pointer to an object in prRetVal from passed position with
        respect to passed pointer to an element.

        \param[in,out] prRetVal In the passed void pointer a found object is returned.
        \param[in] nPosition The position relative to the transferred pRefElement
        \param[in] pRefElement The reference element
        \return A return < 0 indicates an error. A return == #RANGE
          indicates the end of the list.
      \endif
    
      \if german
        Liefert einen rohen Zeiger auf ein Objekt in prRetVal aus übergebener
        Position in Bezug auf übergebenen Zeiger auf ein Element.

        \param[in,out] prRetVal Im übergebenen rohen Pointer wird ein gefundenes
          Objekt geliefert.
        \param[in] nPosition Die Position rellativ zum übergebenen pRefElement
        \param[in] pRefElement Das Referenzelement
        \return Eine Rückgabe < 0 zeigt einen Fehler an. Eine Rückgabe == #RANGE
          zeigt das Ende der Liste an.
      \endif
      */
      int get( void *& prRetVal, std::size_t nPosition, DVElement * pRefElement );
      //-----------------------------------------------------------------------
      /*!
      \if english
        This method inserts the void pointer to a data object in the list. The current
        element is set to the new element.

        If the list was instantiated with parameter #UNIQUE and the data object is
        already in the list, it is not inserted.
        \param[in] pDataObject Is the void pointer to the data object.
        \param[in] eWhere Indicates where the data object is to be inserted in the
          list. Possible parameters are #InsertHEAD, #InsertLAST and
          #InsertCURRENT .
        \param[in] eBeforeBehind Indicates whether the new element is placed before
          or after the element specified in the eWhere parameter. Possible
          parameters are #BEFORE and #BEHIND.
        \return A return value < 0 indicates an error. The following errors are
          possible: #PASSED_NO_OBJECT,  #INSTANTIATE_DVELEMENT and
          #OBJECT_ALREADY_AVAILABLE  (only at #UNIQUE).
      \endif
    
      \if german
        Diese Methode fügt den rohen Zeiger auf ein Datenobjekt in die Liste ein.
        Das aktuelle Element wird auf das neue Element gesetzt.

        Sollte die Liste mit Parameter #UNIQUE instanziiert worden sein und das
        Datenobjekt schon in der Liste sein, wird es nicht eingefügt.
        \param[in] pDataObject Ist der rohe Zeiger auf das Datenobjekt.
        \param[in] eWhere Zeigt an, wohin das Datenobjekt in die Liste eingefügt
          werden soll. Mögliche Parameter sind #InsertHEAD, #InsertLAST und
          #InsertCURRENT .
        \param[in] eBeforeBehind Gibt an, ob das neue Element vor oder hinter das,
          im Parameter eWhere angegebene Element, gesetzt wird. Mögliche Parameter
          sind #BEFORE und #BEHIND.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
        Folgende Fehler sind möglich: #PASSED_NO_OBJECT,
        #INSTANTIATE_DVELEMENT und #OBJECT_ALREADY_AVAILABLE
        (nur bei #UNIQUE).
      \endif
      */
      virtual int insert( void * pDataObject,
                          InsertPosition eWhere = InsertLAST,
                          VH eBeforeBehind = BEHIND ) override;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Removes the element specified as parameter from the list.

        If a subsequent element exists, it becomes the current element, otherwise
        the previous element, if it exists.
        \param[in] eElementPos Possible values are #FIRST_POS, #LAST_POS and #CURRENT.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Entfernt das Element, welches als Parameter angegeben ist, aus der Liste.

        Wenn ein nachfolgendes Element vorhanden ist, wird dieses zum aktuellen Element,
        ansonsten das vorherige Element, wenn dieses vorhanden ist.
        \param[in] eElementPos Mögliche Werte sind #FIRST_POS, #LAST_POS und #CURRENT.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
      \endif
      */
      int remove( ElementPosition eElementPos = CURRENT_POS );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Removes all elements from the list.
      \endif
    
      \if german
        Entfernt alle Elemente aus Liste.
      \endif
      */
      virtual void removeAll() override;
      //-----------------------------------------------------------------------
      /*!
      \if english
        This method allows you to cache or restore the current pointer.
        \sa #m_pCurrentDVElement element and #m_pTempCurrentDVElement element.
        \param[in] eWhich Specifies whether the pointer to the current element
          is cached (M) or restored (RM).
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Diese Methode ermöglicht das Zwischenspeichern oder Restaurieren des
        aktuellen Zeigers.
        \sa #m_pCurrentDVElement und #m_pTempCurrentDVElement.
        \param[in] eWhich Gibt an, ob der Zeiger auf das aktuelle Element
          zwischengespeichert (M) oder restauriert (RM) wird.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
      \endif
      */
      int currentElementCache( ElementCache eWhich );
      //-----------------------------------------------------------------------
      /*!
      \if english
        If there is a PtlDV element with the passed pointer in the list, the
        current pointer is set to the PtlDV element, otherwise the current pointer
        remains what it was.
        \param[in] pDataObject The data object you are looking for
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Wenn DVElement mit übergebener Zeiger in der Liste vorhanden ist, wird der
        aktuelle Zeiger auf das DVElement gesetzt, ansonsten bleibt der aktuelle
        Zeiger was er war.
        \param[in] pDataObject Das gesuchte Datenobjekt
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
      \endif
      */
      int setCurrentPosition( void * pDataObject );
      //-----------------------------------------------------------------------

      //+ ============ Operatoren ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Assignment operator=

        During an assignment, all own DVElements are removed and then all
        data objects from the assigning VDVList are transferred to new own
        DVElements.
        \param[in] rList The list to be assigned.

        It should be called after call getErrorCode().
      \endif
    
      \if german
        Zuweisungsoperator=

        Bei einer Zuweisung werden alle DVElement entfernt und dann alle Datenobjekte
        aus der zuweisenden VDVList in neue eigene DVElement übernommen.
        \param[in] rList Die zuzuweisende Liste.

        Es sollte nach Aufruf getErrorCode() aufgerufen werden.
      \endif
      */
      VDVList & operator= ( const VDVList& rList );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
} // namespace ptl
#endif
